#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>

/*

     design for graphically creating input form structures

     needs xinput.c and xinput.h common X input forms code
     also needs menusub.c and menusub.h

     (based on gfx)
     
     last modified
     	
     	2/24/97 updated to new ximage routines
     		changed to pop-up menu on main window click
     		fixed delete bug
     		added apply to edit window
     		removed the menu window code(replaced by pop-up menu)
     		changed to defines for window numbers
     		will take command line filename for save
     		added -v debugging output switch
     	2/26/97
     		added code to keep track of window positions

*/


extern char *   getenv(/* const char *name */);

/* X include files */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#ifdef DVX
    #include <X11/cursorfo.h>
#else
    #include <X11/cursorfont.h>
#endif

/* declarations */
char title[50];

/* X declarations */
Display *mydisplay;
Window mywindow;
Window root;
GC mygc;
XEvent myevent;
KeySym mykey;
XSizeHints myhint;
XWMHints *mywmhint = NULL;
XClassHint *myclasshint = NULL;
XTextProperty mytitle;
int myscreen;
unsigned long myforeground, mybackground;
Colormap mycolormap;
Cursor mycursor;
Visual myvisual;
XColor mycolor_def;
XColor mycolor_rgb;
Font myfont;

unsigned int xsize,ysize,xl,xr,yu,yb;
unsigned int line_width,border_width,depth;
int line_style,cap_style,join_style;
unsigned long mycolors[20];
int num_mycolors;
int fheight=13;       /* should get these from x font */
int fwidth=8;         /* set for 8x13 font set below */
char filename[50];

#include "../xinput/xinput.h"
#include "../xinput/menusub.h"

#define MAIN_WIN 0
#define BUTTON_WIN 1
#define TEXTBOX_WIN 2
#define LISTBOX_WIN 3
#define EDIT_WIN 4

#define NUM_WINS 5

int main_xcen,main_ycen;

int debug;

/* Hack to account for WM borders in window position */
#define WM_TITLEBAR_HEIGHT 21
#define WM_BORDER_WIDTH 4

main(argc,argv)
int argc;
char **argv;
{
int i;
char txt[10];
int done;

extern int    optind;  /* for command line options parsing */
extern char   *optarg;
char  *cmd;
char  *display = NULL;
int   c;
char *geo = NULL;
char *defgeo = "200x50+200+200";  /* default geometry for the
graphics window */
int bw = 2;  /* border width, gets ignored by window manager */
int x,y,xwidth,xheight;
char hostname[50], temp[50];
int hostlen=50;
char *titleptr;

char input[100];
int row,col;

  cmd = argv[0];
  while ((c = getopt(argc, argv, "hd:g:v")) != EOF)
    switch(c)   {

    case 'd':
      display = optarg;
      break;
    case 'g':
      geo = optarg;
      break;
    case 'v':
    	debug = 1;
    	break;
    default:
    case 'h':
      usage(cmd);
      exit(0);
    }

/* get command line file name if there */

    if( optind < argc )
    {
        strcpy(filename,argv[optind]);
    }
    else
    {
    	strcpy(filename,"");
    }


    if ((mydisplay = XOpenDisplay(display)) == NULL)
    {
    	fprintf(stderr, "Can\'t open display: %s\n", display );
    	exit(0);
    }


        /* initialization */

        myscreen = DefaultScreen(mydisplay);

	/* setups for xinput and menusub routines */
	
	xi_init(mydisplay,myscreen,NULL,NULL,NULL);
	ms_init(mydisplay,myscreen,NULL,NULL,NULL);
	
        XGeometry (mydisplay, myscreen, geo, defgeo, bw, 1, 1, 0, 0,
                        &x, &y, &xwidth,&xheight);
        xsize=xwidth;
        ysize=xheight;

        set_in_wins();
        set_menu();

        xsize = inputwin[MAIN_WIN].xsize;
        ysize = inputwin[MAIN_WIN].ysize;

        /* default pixel values */
        mybackground = WhitePixel(mydisplay,myscreen);
        myforeground = BlackPixel(mydisplay,myscreen);

        /* default program-specified window position and size */
        myhint.x=x;
        myhint.y=y;
        myhint.flags = PPosition | PSize;
        myhint.width = xsize;
        myhint.height = ysize;

        gethostname(hostname,hostlen);
        strcpy(title,"Design Form ");
        if(strlen(hostname) !=0 )
        {
                strcpy(temp,"");
                sscanf(hostname,"%[^.]",temp);
                strcat(title,temp);
        }

        /* window creation */
        mywindow = XCreateSimpleWindow(mydisplay,
                DefaultRootWindow(mydisplay),
                myhint.x, myhint.y, myhint.width, myhint.height,
                5, myforeground, mybackground);

        titleptr=title;
        XStringListToTextProperty(&titleptr,1,&mytitle);

        /* default program-specified window position and size */
        myhint.x=x;
        myhint.y=y;
        myhint.flags = PPosition | PSize ;
        myhint.width = xsize;
        myhint.height = ysize;
        
        main_xcen = x + xsize/2;
        main_ycen = y + ysize/2;

        XSetWMProperties(mydisplay,mywindow,&mytitle,&mytitle,argv,argc,
                &myhint,mywmhint,myclasshint);

        /* GC creation and initialization */
        mygc= XCreateGC(mydisplay, mywindow, 0, NULL);
        XSetBackground(mydisplay, mygc, mybackground);
        XSetForeground(mydisplay, mygc, myforeground);

        myfont = XLoadFont(mydisplay,"8x13");
        XSetFont(mydisplay,mygc,myfont);

        /* input event selection */
        XSelectInput(mydisplay,mywindow,
             ButtonPressMask | KeyPressMask | ExposureMask |
             StructureNotifyMask);

        /* window mapping */
        XMapRaised(mydisplay, mywindow);

        /* line attributes */
        line_width=1;
        line_style=LineSolid;
        cap_style=CapButt;
        join_style=JoinMiter;
        XSetLineAttributes(mydisplay,mygc,line_width,line_style,cap_style,join_style);

        get_size();

        set_colors();


        /* Main event-reading loop */
        done = 0;
        while( done == 0) {
                /* read the next event */
                XNextEvent(mydisplay, &myevent);
                
                if(myevent.xany.window == mywindow)
                {
                switch(myevent.type){
                        /* repaint window on expose events */
                        case Expose:

                                if(myevent.xexpose.count == 0)
                                {
                                        redraw(0);
                                }

                        break;

                        case ConfigureNotify:
                                set_box(myevent);
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                             XRefreshKeyboardMapping(&myevent.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:
                                    col=myevent.xmotion.x;
                                    row=myevent.xmotion.y;
                                    which_control(row,col,myevent.xbutton.x_root,myevent.xbutton.y_root);

                        break;

                        /* process keyboard input, can be done nicer */
                        case KeyPress:
                                i = XLookupString( &myevent.xkey, txt, 10, &mykey, 0);
                                if( i == 1 && txt[0] == 'q') 
                                {
                                	if(yes_no_window(main_xcen,main_ycen,"Quit?",NO) > 0)
		                                done = 1;
		                 }
                                if( i == 1 && txt[0] == 's')
                                {
                                               save_inwin_struct(MAIN_WIN);
                                }
                        break;
                } /* switch(myevent.type) */
                } /* if mywindow */
        } /* while(done == 0) */


        design_exit();
}

save_inwin_struct(int win)
{
int i,j;
FILE *file;
char temp[100],tname[100];

	   if(strlen(filename) > 0)
	   {
	   	strcpy(tname,filename);
	   }
	   else
	   {
	   	strcpy(tname,"struct.dat");
	   }
	   
	   sprintf(temp,"Save structure 0 to %s?",tname);

	   if(yes_no_window(main_xcen,main_ycen,temp,YES) != YES)
	  	return;
	  	
	   if((file = fopen(tname,"r")) != NULL)
	   {
	   	fclose(file);
	   	if(yes_no_window(main_xcen,main_ycen,"File Exists, Overwrite?",NO) != YES)
	   		return;
	   }

	   file = fopen(tname,"w");
           
           if(file == NULL)
           {
           	info_window(main_xcen,main_ycen,"Error on file open");
           	return;
           }

           fprintf(file,"/* \n");
           fprintf(file,"\tWindow %i definitions of inputwin structure \n",win);
           fprintf(file,"*/ \n");

           fprintf(file,"\twin = %i;\n",win);
           fprintf(file,"\n");
           fprintf(file,"\tinputwin[win].xsize = %i;\n",inputwin[win].xsize);
           fprintf(file,"\tinputwin[win].ysize = %i;\n",inputwin[win].ysize);
           fprintf(file,"\tinputwin[win].num_inputs = %i;\n",inputwin[win].num_inputs);
           fprintf(file,"\tinputwin[win].num_lists = %i;\n",inputwin[win].num_lists);
           fprintf(file,"\tinputwin[win].num_buttons = %i;\n",inputwin[win].num_buttons);
           fprintf(file,"\tinputwin[win].button_width = %i;\n",inputwin[win].button_width);
           fprintf(file,"\tinputwin[win].button_height = %i;\n",inputwin[win].button_height);
           fprintf(file,"\tstrcpy(inputwin[win].title,\"%s\");\n",inputwin[win].title);
           fprintf(file,"\n");
           fprintf(file,"\tinputwin[win].num_inputs = %i;\n",inputwin[win].num_inputs);
           if(inputwin[win].num_inputs > 0)
           {
                for(i=0;i<inputwin[win].num_inputs;i++)
                {
                        fprintf(file,"\n");
                        fprintf(file,"\t\tinputwin[win].inputs[%i].row = %i;\n",i,inputwin[win].inputs[i].row);
                        fprintf(file,"\t\tinputwin[win].inputs[%i].col = %i;\n",i,inputwin[win].inputs[i].col);
                        fprintf(file,"\t\tinputwin[win].inputs[%i].maxlen = %i;\n",i,inputwin[win].inputs[i].maxlen);
                        fprintf(file,"\t\tinputwin[win].inputs[%i].disable = 0;\n",i);
                        fprintf(file,"\t\tinputwin[win].inputs[%i].noedit = 0;\n",i);
                        fprintf(file,"\t\tstrcpy(inputwin[win].inputs[%i].def,\"%s\");\n",i,inputwin[win].inputs[i].def);
                        fprintf(file,"\t\tstrcpy(inputwin[win].inputs[%i].caption,\"%s\");\n",i,inputwin[win].inputs[i].caption);
                }
           }
           fprintf(file,"\n");
           fprintf(file,"\tinputwin[win].num_buttons = %i;\n",inputwin[win].num_buttons);
           if(inputwin[win].num_buttons > 0)
           {
           	fprintf(file,"\tinputwin[win].default_button = -1;\n");
                for(i=0;i<inputwin[win].num_buttons;i++)
                {
                        fprintf(file,"\n");
                        fprintf(file,"\t\tinputwin[win].buttons[%i].row = %i;\n",i,inputwin[win].buttons[i].row);
                        fprintf(file,"\t\tinputwin[win].buttons[%i].col = %i;\n",i,inputwin[win].buttons[i].col);
                        fprintf(file,"\t\tinputwin[win].buttons[%i].disable  = 0;\n",i);
                        fprintf(file,"\t\tstrcpy(inputwin[win].buttons[%i].text,\"%s\");\n",i,inputwin[win].buttons[i].text);
                }
           }
           fprintf(file,"\n");
           fprintf(file,"\tinputwin[win].num_lists = %i;\n",inputwin[win].num_lists);
           if(inputwin[win].num_lists > 0)
           {
                for(i=0;i<inputwin[win].num_lists;i++)
                {
                        fprintf(file,"\n");
                        fprintf(file,"\t\tinputwin[win].lists[%i].row = %i;\n",i,inputwin[win].lists[i].row);
                        fprintf(file,"\t\tinputwin[win].lists[%i].col = %i;\n",i,inputwin[win].lists[i].col);
                        fprintf(file,"\t\tinputwin[win].lists[%i].maxlen = %i;\n",i,inputwin[win].lists[i].maxlen);
                        fprintf(file,"\t\tinputwin[win].lists[%i].disable = 0;\n",i);
                        fprintf(file,"\t\tinputwin[win].lists[%i].numeric = 0;\n",i);
                        fprintf(file,"\t\tinputwin[win].lists[%i].def = %i;\n",i,inputwin[win].lists[i].item);
                        fprintf(file,"\t\tstrcpy(inputwin[win].lists[%i].caption,\"%s\");\n",i,inputwin[win].lists[i].caption);
                        fprintf(file,"\n");
                        fprintf(file,"\t\tinputwin[win].lists[%i].list_len = %i;\n",i,inputwin[win].lists[i].list_len);
                        for(j=0;j<inputwin[win].lists[i].list_len;j++)
                        {
                              fprintf(file,"\t\t\tstrcpy(inputwin[win].lists[%i].text[%i],\"%s\");\n",i,j,inputwin[win].lists[i].text[j]);

                        }

                }
          }

        fclose(file);

}


/*

****************************************************************

   Input Window in X routines: Text and list boxes and buttons

****************************************************************

*/
set_in_wins()
{
int numwin,i,j,win;

        numwin = NUM_WINS;

/* Design window settings  -- all other entries for this window 
   are user inputs -- and it is also resizable
*/

        inputwin[MAIN_WIN].xsize = 500;
        inputwin[MAIN_WIN].ysize = 400;
        inputwin[MAIN_WIN].num_inputs = 0;
        inputwin[MAIN_WIN].num_buttons=0;
        inputwin[MAIN_WIN].num_lists = 0;
        inputwin[MAIN_WIN].button_width = BUTTONWIDTH;
        inputwin[MAIN_WIN].button_height = BUTTONHEIGHT;
        inputwin[MAIN_WIN].default_button = -1;


/* button form window size (fixed): Window BUTTON_WIN */

        inputwin[BUTTON_WIN].xsize = 400;
        inputwin[BUTTON_WIN].ysize = 100;

/* input window title */
        strcpy(inputwin[BUTTON_WIN].title,"Button");

/* text input boxes section: Window 2 */
        inputwin[BUTTON_WIN].num_inputs = 1;
        /* input box 1 */
                strcpy(inputwin[BUTTON_WIN].inputs[0].def,"Ok");
                inputwin[BUTTON_WIN].inputs[0].row = 40;
                inputwin[BUTTON_WIN].inputs[0].col = 125;
                inputwin[BUTTON_WIN].inputs[0].maxlen = 30;
                strcpy(inputwin[BUTTON_WIN].inputs[0].caption,"Button Text");

/* buttons section */
        inputwin[BUTTON_WIN].num_buttons=2;
        inputwin[BUTTON_WIN].button_width = BUTTONWIDTH;
        inputwin[BUTTON_WIN].button_height = BUTTONHEIGHT;
        inputwin[BUTTON_WIN].default_button = 0;

        /* button  1 */
                inputwin[BUTTON_WIN].buttons[0].col = 75;
                inputwin[BUTTON_WIN].buttons[0].row = 70;
                strcpy(inputwin[BUTTON_WIN].buttons[0].text,"Ok");

        /* button  2 */
                inputwin[BUTTON_WIN].buttons[1].col = 275;
                inputwin[BUTTON_WIN].buttons[1].row = 70;
                strcpy(inputwin[BUTTON_WIN].buttons[1].text,"Cancel");


/* list boxes section*/

        inputwin[BUTTON_WIN].num_lists = 0;

/* textform window size (fixed): Window TEXTBOX_WIN */

        inputwin[TEXTBOX_WIN].xsize = 350;
        inputwin[TEXTBOX_WIN].ysize = 200;

/* input window title */
        strcpy(inputwin[TEXTBOX_WIN].title,"Text Box");

/* text input boxes section */
        inputwin[TEXTBOX_WIN].num_inputs = 2;
        /* input box 1 */
                strcpy(inputwin[TEXTBOX_WIN].inputs[0].def,"Text");
                inputwin[TEXTBOX_WIN].inputs[0].row = 50;
                inputwin[TEXTBOX_WIN].inputs[0].col = 100;
                inputwin[TEXTBOX_WIN].inputs[0].maxlen = 30;
                strcpy(inputwin[TEXTBOX_WIN].inputs[0].caption,"Caption");

        /* input box 1 */
                strcpy(inputwin[TEXTBOX_WIN].inputs[1].def,"20");
                inputwin[TEXTBOX_WIN].inputs[1].row = 100;
                inputwin[TEXTBOX_WIN].inputs[1].col = 100;
                inputwin[TEXTBOX_WIN].inputs[1].maxlen = 5;
                strcpy(inputwin[TEXTBOX_WIN].inputs[1].caption,"Max length");

/* buttons section */
        inputwin[TEXTBOX_WIN].num_buttons=2;
        inputwin[TEXTBOX_WIN].button_width = BUTTONWIDTH;
        inputwin[TEXTBOX_WIN].button_height = BUTTONHEIGHT;
        inputwin[TEXTBOX_WIN].default_button = 0;

        /* button  1 */
                inputwin[TEXTBOX_WIN].buttons[0].col = 50;
                inputwin[TEXTBOX_WIN].buttons[0].row = 150;
                strcpy(inputwin[TEXTBOX_WIN].buttons[0].text,"Ok");

        /* button  2 */
                inputwin[TEXTBOX_WIN].buttons[1].col = 250;
                inputwin[TEXTBOX_WIN].buttons[1].row = 150;
                strcpy(inputwin[TEXTBOX_WIN].buttons[1].text,"Cancel");


/* list boxes section */
        inputwin[TEXTBOX_WIN].num_lists = 0;

/* list form window size (fixed): Window LISTBOX_WIN */

        inputwin[LISTBOX_WIN].xsize = 350;
        inputwin[LISTBOX_WIN].ysize = 250;

/* input window title */
        strcpy(inputwin[LISTBOX_WIN].title,"List Box");

/* text input boxes section */
        inputwin[LISTBOX_WIN].num_inputs = 3;
        /* input box 1 */
                strcpy(inputwin[LISTBOX_WIN].inputs[0].def,"Text");
                inputwin[LISTBOX_WIN].inputs[0].row = 50;
                inputwin[LISTBOX_WIN].inputs[0].col = 100;
                inputwin[LISTBOX_WIN].inputs[0].maxlen = 30;
                strcpy(inputwin[LISTBOX_WIN].inputs[0].caption,"Caption");

        /* input box 2 */
                strcpy(inputwin[LISTBOX_WIN].inputs[1].def,"20");
                inputwin[LISTBOX_WIN].inputs[1].row = 100;
                inputwin[LISTBOX_WIN].inputs[1].col = 100;
                inputwin[LISTBOX_WIN].inputs[1].maxlen = 5;
                strcpy(inputwin[LISTBOX_WIN].inputs[1].caption,"Max length");

        /* input box 3 */
                strcpy(inputwin[LISTBOX_WIN].inputs[2].def,"5");
                inputwin[LISTBOX_WIN].inputs[2].row = 150;
                inputwin[LISTBOX_WIN].inputs[2].col = 100;
                inputwin[LISTBOX_WIN].inputs[2].maxlen = 5;
                strcpy(inputwin[LISTBOX_WIN].inputs[2].caption,"Num items");

/* buttons section */
        inputwin[LISTBOX_WIN].num_buttons=2;
        inputwin[LISTBOX_WIN].button_width = BUTTONWIDTH;
        inputwin[LISTBOX_WIN].button_height = BUTTONHEIGHT;
        inputwin[LISTBOX_WIN].default_button = 0;

        /* button  1 */
                inputwin[LISTBOX_WIN].buttons[0].col = 50;
                inputwin[LISTBOX_WIN].buttons[0].row = 200;
                strcpy(inputwin[LISTBOX_WIN].buttons[0].text,"Ok");

        /* button  2 */
                inputwin[LISTBOX_WIN].buttons[1].col = 250;
                inputwin[LISTBOX_WIN].buttons[1].row = 200;
                strcpy(inputwin[LISTBOX_WIN].buttons[1].text,"Cancel");


/* list boxes section*/
        inputwin[LISTBOX_WIN].num_lists = 0;


/* edit form window size (fixed): Window EDIT_WIN */

        inputwin[EDIT_WIN].xsize = 300;
        inputwin[EDIT_WIN].ysize = 200;

/* input window title */
        strcpy(inputwin[EDIT_WIN].title,"Edit");

/* text input boxes section */
        inputwin[EDIT_WIN].num_inputs = 2;
        /* input box 1 */
                strcpy(inputwin[EDIT_WIN].inputs[0].def,"0");
                inputwin[EDIT_WIN].inputs[0].row = 50;
                inputwin[EDIT_WIN].inputs[0].col = 125;
                inputwin[EDIT_WIN].inputs[0].maxlen = 5;
                strcpy(inputwin[EDIT_WIN].inputs[0].caption,"Row");

        /* input box 2 */
                strcpy(inputwin[EDIT_WIN].inputs[1].def,"0");
                inputwin[EDIT_WIN].inputs[1].row = 100;
                inputwin[EDIT_WIN].inputs[1].col = 125;
                inputwin[EDIT_WIN].inputs[1].maxlen = 5;
                strcpy(inputwin[EDIT_WIN].inputs[1].caption,"Column");

/* buttons section */
        inputwin[EDIT_WIN].num_buttons = 3;
        inputwin[EDIT_WIN].button_width = BUTTONWIDTH;
        inputwin[EDIT_WIN].button_height = BUTTONHEIGHT;
        inputwin[EDIT_WIN].default_button = 0;

        /* button  1 */
                inputwin[EDIT_WIN].buttons[0].col = 50;
                inputwin[EDIT_WIN].buttons[0].row = 150;
                strcpy(inputwin[EDIT_WIN].buttons[0].text,"Ok");

        /* button  2 */
                inputwin[EDIT_WIN].buttons[1].col = 250;
                inputwin[EDIT_WIN].buttons[1].row = 150;
                strcpy(inputwin[EDIT_WIN].buttons[1].text,"Delete");
                
        /* button  3 */
                inputwin[EDIT_WIN].buttons[2].col = 150;
                inputwin[EDIT_WIN].buttons[2].row = 150;
                strcpy(inputwin[EDIT_WIN].buttons[2].text,"Apply");


/* list boxes section */
        inputwin[EDIT_WIN].num_lists = 0;

/************
             set text to default values

************/

        for(i=0;i<numwin;i++)
        {
           if(inputwin[i].num_inputs > 0)
           {
           for(j=0;j<inputwin[i].num_inputs;j++)
           {
                strcpy(inputwin[i].inputs[j].text,inputwin[i].inputs[j].def);
           }
           }
        }
}

which_control(int inrow,int incol,int xroot,int yroot)
{
/*
                asks if button, text or list box to add

                opens a second window for input using inputwin structure

                draws buttons and text input/list boxes

                returns the button pressed and the updated text (in place)
                      and the updated list items

                includes event loop with redraw and resize

                needs the redraw for the main window added.

                could add keyboard short cuts

                uses redraw_inwin() for its window redraw

*/

int done,i,in,q,win;
int col,row,drow,dcol;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;

        if(inputwin[MAIN_WIN].num_inputs > 0)
        {
           for(i=0;i<inputwin[MAIN_WIN].num_inputs;i++)
           {
                drow=abs(inrow - inputwin[MAIN_WIN].inputs[i].row);
                dcol=abs(incol - inputwin[MAIN_WIN].inputs[i].col);
                if( (drow < BUTTONHEIGHT/2) && (dcol < BUTTONWIDTH/2))
                {
                        edit_form(i,1);
                        return;
                }
           }
        }

        if(inputwin[MAIN_WIN].num_buttons > 0)
        {
           for(i=0;i<inputwin[MAIN_WIN].num_buttons;i++)
           {
                drow=abs(inrow - inputwin[MAIN_WIN].buttons[i].row);
                dcol=abs(incol - inputwin[MAIN_WIN].buttons[i].col);
                if( (drow < BUTTONHEIGHT/2) && (dcol < BUTTONWIDTH/2))
                {
                        edit_form(i,2);
                        return;
                }
           }
        }

        if(inputwin[MAIN_WIN].num_lists > 0)
        {
           for(i=0;i<inputwin[MAIN_WIN].num_lists;i++)
           {
                drow=abs(inrow - inputwin[MAIN_WIN].lists[i].row);
                dcol=abs(incol - inputwin[MAIN_WIN].lists[i].col);
                if( (drow < BUTTONHEIGHT/2) && (dcol < BUTTONWIDTH/2))
                {
                        edit_form(i,3);
                        return;
                }
           }
        }
        
        /* if not a click on a existing object, display menu */
        
        do_menu(menu_window(xroot,yroot,0),inrow,incol);
        
        return;

}

set_menu()
{
	menu_info[0].num_items = 5;
	
	strcpy(menu_info[0].label[0],"Button");
	menu_info[0].disable[0] = NO;
	strcpy(menu_info[0].label[1],"Text Box");
	menu_info[0].disable[1] = NO;
	strcpy(menu_info[0].label[2],"List Box");
	menu_info[0].disable[2] = NO;
	strcpy(menu_info[0].label[3],"Save");
	menu_info[0].disable[3] = NO;
	strcpy(menu_info[0].label[4],"Quit");
	menu_info[0].disable[4] = NO;
	
}

do_menu(int q,int inrow,int incol)
{
                                    if(q == 0)
                                    {
                                       /* code for button */
                                       button_form(inrow,incol);

                                    }
                                    if(q == 1)
                                    {
                                       /* code for text */
                                        text_form(inrow,incol);

                                    }
                                    if(q == 2)
                                    {
                                       /* code for list */
                                         list_form(inrow,incol);
                                    }
                                    if(q == 3)
                                    {
                                          save_inwin_struct(0);
                                    }
                                    if(q == 4)
                                    {
                                        if(yes_no_window(main_xcen,main_ycen,"Quit?",NO) == YES )
                                    		design_exit();
                                    }

}

button_form(int inrow, int incol)
{
/*
                gets button text

                opens a second window for input using inputwin structure

                draws buttons and text input/list boxes

                returns the button pressed and the updated text (in place)
                      and the updated list items

                includes event loop with redraw and resize

                needs the redraw for the main window added.

                could add keyboard short cuts

                uses redraw_inwin() for its window redraw

*/

int done,i,in,q,win;
static int xpos=-1;
static int ypos= -1;
int col,row;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;

        win = BUTTON_WIN; /* window number in the structure */
        
        if(xpos == -1 && ypos == -1)
        {
        	xpos = main_xcen - inputwin[win].xsize/2;
        	ypos = main_ycen - inputwin[win].ysize/2;
        	if(debug)
        		printf("button init pos: %i %i \n",xpos,ypos);
	}        	

         open_input_window(&my2window,&my2gc,xpos,ypos,inputwin[win].xsize,
                           inputwin[win].ysize,inputwin[win].title);


        /* event-reading loop to get cursor position */
        done = 0;
        while( done == 0) {
            /* read the next event */
            XNextEvent(mydisplay, &my2event);
            /* redraw main window */
            if(my2event.xany.window == mywindow)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                              {
                                        redraw(0);
                              }

                        break;

                        case ConfigureNotify:
                                set_box(my2event);
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                              XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* mouse-button presses in main beep */
                        case ButtonPress:
                                XBell(mydisplay,100);
                        break;

                        /* keyboard input in main is a beep */
                        case KeyPress:
                                XBell(mydisplay,100);
                        break;
                }
            }
            /* events for input window */
            if(my2event.xany.window == my2window)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                                        redraw_inwin(my2window,my2gc,win);

                        break;

                        case LeaveNotify:
                               redraw_inwin(my2window,my2gc,win);
                               q = -1;
                        break;
                        
                        case ConfigureNotify:
				save_position(my2event,&xpos,&ypos);
                        break;

                        case MotionNotify:
                                if(q != -1)
                                {
                                        if(q != which_button(my2event.xmotion.x,my2event.xmotion.y,win) )
                                        {
                                             redraw_inwin(my2window,my2gc,win);
                                             q = -1;
                                        }
                                  }
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                        	XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:

                                    col=my2event.xmotion.x;
                                    row=my2event.xmotion.y;

                                    /* check input boxes */

                                    in = check_input(win,row,col);
                                    if( in > -1 )
                                    {
                                         if(get_input(my2window,my2gc,in,win) >= 0 )
                                         {
                                        /* code to do on input change goes next */


                                          }
                                          else
                                          {
                                               redraw_inwin(my2window,my2gc,win);
                                          }
                                      }

                                      q=check_button(my2window,my2gc,col,row,win);

                                      if(q == 0)
                                      {
                                       /* code for button 0 goes next */

                                           add_button(win,inrow,incol);
                                           redraw(1);
                                           done =1;

                                      }
                                      if(q == 1)
                                      {
                                       /* code for button 0 goes next */

                                            done = 1;
                                      }



                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &mykey, 0);
                                /* if form key pressed code is needed it goes next */

                                if( mykey == XK_Escape ) done = 1;

                                if( (mykey == XK_Return) || (mykey == XK_KP_Enter) || (mykey == XK_Linefeed))
                                {
                                           add_button(win,inrow,incol);
                                           redraw(1);
                                           done = 1;
                                }

                        break;
                } /* switch(my2event.type) */
            } /* if my window */
        } /* while(done == 0) */

        close_input_window(my2window,my2gc);

        return(done);
}

add_button(int win,int inrow, int incol)
{
int i;
      i=inputwin[MAIN_WIN].num_buttons;
      inputwin[MAIN_WIN].buttons[i].col=incol;
      inputwin[MAIN_WIN].buttons[i].row=inrow;
      strcpy(inputwin[MAIN_WIN].buttons[i].text,inputwin[win].inputs[0].text);
      inputwin[MAIN_WIN].num_buttons=i+1;

}

text_form(int inrow, int incol)
{
/*
                gets text box settings

                opens a second window for input using inputwin structure

                draws buttons and text input/list boxes

                returns the button pressed and the updated text (in place)
                      and the updated list items

                includes event loop with redraw and resize

                needs the redraw for the main window added.

                could add keyboard short cuts

                uses redraw_inwin() for its window redraw

*/

int done,i,in,q,win;
static int xpos=-1;
static int ypos= -1;
int col,row;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;

        win = TEXTBOX_WIN; /* window number in the structure */
        
        if(xpos == -1 && ypos == -1)
        {
        	xpos = main_xcen - inputwin[win].xsize/2;
        	ypos = main_ycen - inputwin[win].ysize/2;
        	if(debug)
        		printf("button init pos: %i %i \n",xpos,ypos);
	}        	

         open_input_window(&my2window,&my2gc,xpos,ypos,inputwin[win].xsize,
                           inputwin[win].ysize,inputwin[win].title);


        /* event-reading loop to get cursor position */
        done = 0;
        while( done == 0) {
            /* read the next event */
            XNextEvent(mydisplay, &my2event);
            /* redraw main window */
            if(my2event.xany.window == mywindow)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                              {
                                        redraw(0);
                              }

                        break;

                        case ConfigureNotify:
                                set_box(my2event);
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                              XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* mouse-button presses in main beep */
                        case ButtonPress:
                                XBell(mydisplay,100);
                        break;

                        /* keyboard input in main is a beep */
                        case KeyPress:
                                XBell(mydisplay,100);
                        break;
                }
            }
            /* events for input window */
            if(my2event.xany.window == my2window)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                                        redraw_inwin(my2window,my2gc,win);

                        break;

                        case LeaveNotify:
                               redraw_inwin(my2window,my2gc,win);
                               q = -1;
                        break;

                        case ConfigureNotify:
				save_position(my2event,&xpos,&ypos);
                        break;

                        case MotionNotify:
                                if(q != -1)
                                {
                                        if(q != which_button(my2event.xmotion.x,my2event.xmotion.y,win) )
                                        {
                                             redraw_inwin(my2window,my2gc,win);
                                             q = -1;
                                        }
                                  }
                        break;


                        /* process keyboard mapping changes */
                        case MappingNotify:
                             XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:

                                    col=my2event.xmotion.x;
                                    row=my2event.xmotion.y;

                                    /* check input boxes */

                                    in = check_input(win,row,col);
                                    if( in > -1 )
                                    {
                                         if(get_input(my2window,my2gc,in,win) >= 0 )
                                         {
                                        /* code to do on input change goes next */


                                          }
                                          else
                                          {
                                               redraw_inwin(my2window,my2gc,win);
                                          }
                                      }

                                      q=check_button(my2window,my2gc,col,row,win);

                                      if(q == 0)
                                      {
                                       /* code for button 0 goes next */

                                           add_input(win,inrow,incol);
                                           redraw(1);
                                           done =1;

                                      }
                                      if(q == 1)
                                      {
                                       /* code for button 0 goes next */

                                            done = 1;
                                      }



                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &mykey, 0);
                                /* if form key pressed code is needed it goes next */
                                if( mykey == XK_Escape ) done = 1;

                                if( (mykey == XK_Return) || (mykey == XK_KP_Enter) || (mykey == XK_Linefeed))
                                {
                                           add_input(win,inrow,incol);
                                           redraw(1);
                                           done = 1;
                                }

                        break;
                } /* switch(my2event.type) */
            } /* if my window */
        } /* while(done == 0) */

        close_input_window(my2window,my2gc);

        return(done);
}

add_input(int win,int inrow, int incol)
{
int i;
      i=inputwin[MAIN_WIN].num_inputs;
      inputwin[MAIN_WIN].inputs[i].col=incol;
      inputwin[MAIN_WIN].inputs[i].row=inrow;
      inputwin[MAIN_WIN].inputs[i].maxlen=atoi(inputwin[win].inputs[1].text);
      strcpy(inputwin[MAIN_WIN].inputs[i].caption,inputwin[win].inputs[0].text);
      inputwin[MAIN_WIN].num_inputs=i+1;

}

list_form(int inrow, int incol)
{
/*
                gets list box settings

                opens a second window for input using inputwin structure

                draws buttons and text input/list boxes

                returns the button pressed and the updated text (in place)
                      and the updated list items

                includes event loop with redraw and resize

                needs the redraw for the main window added.

                could add keyboard short cuts

                uses redraw_inwin() for its window redraw

*/

int done,i,in,q,win;
static int xpos=-1;
static int ypos= -1;
int col,row;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;

        win = LISTBOX_WIN; /* window number in the structure */
        
        if(xpos == -1 && ypos == -1)
        {
        	xpos = main_xcen - inputwin[win].xsize/2;
        	ypos = main_ycen - inputwin[win].ysize/2;
        	if(debug)
        		printf("button init pos: %i %i \n",xpos,ypos);
	}        	

         open_input_window(&my2window,&my2gc,xpos,ypos,inputwin[win].xsize,
                           inputwin[win].ysize,inputwin[win].title);


        /* event-reading loop to get cursor position */
        done = 0;
        while( done == 0) {
            /* read the next event */
            XNextEvent(mydisplay, &my2event);
            /* redraw main window */
            if(my2event.xany.window == mywindow)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                              {
                                        redraw(0);
                              }

                        break;

                        case ConfigureNotify:
                                set_box(my2event);
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                              XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* mouse-button presses in main beep */
                        case ButtonPress:
                                XBell(mydisplay,100);
                        break;

                        /* keyboard input in main is a beep */
                        case KeyPress:
                                XBell(mydisplay,100);
                        break;
                }
            }
            /* events for input window */
            if(my2event.xany.window == my2window)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                                        redraw_inwin(my2window,my2gc,win);

                        break;

                        case LeaveNotify:
                               redraw_inwin(my2window,my2gc,win);
                               q = -1;
                        break;

                        case ConfigureNotify:
				save_position(my2event,&xpos,&ypos);
                        break;

                        case MotionNotify:
                                if(q != -1)
                                {
                                        if(q != which_button(my2event.xmotion.x,my2event.xmotion.y,win) )
                                        {
                                             redraw_inwin(my2window,my2gc,win);
                                             q = -1;
                                        }
                                  }
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                              XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:

                                    col=my2event.xmotion.x;
                                    row=my2event.xmotion.y;

                                    /* check input boxes */

                                    in = check_input(win,row,col);
                                    if( in > -1 )
                                    {
                                         if(get_input(my2window,my2gc,in,win) >= 0 )
                                         {
                                        /* code to do on input change goes next */


                                          }
                                          else
                                          {
                                               redraw_inwin(my2window,my2gc,win);
                                          }
                                      }

                                      q=check_button(my2window,my2gc,col,row,win);

                                      if(q == 0)
                                      {
                                       /* add a list box */
                                           add_list(win,inrow,incol);
                                           redraw(1);
                                           done =1;

                                      }
                                      if(q == 1)
                                      {
                                       /* code for button 0 goes next */

                                            done = 1;
                                      }



                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &mykey, 0);
                                /* if form key pressed code is needed it goes next */

                                if( mykey == XK_Escape ) done = 1;

                                if( (mykey == XK_Return) || (mykey == XK_KP_Enter) || (mykey == XK_Linefeed))
                                {
                                           add_list(win,inrow,incol);
                                           redraw(1);
                                           done = 1;
                                }

                        break;
                } /* switch(my2event.type) */
            } /* if my window */
        } /* while(done == 0) */

        close_input_window(my2window,my2gc);

        return(done);
}

add_list(int win,int inrow, int incol)
{
int i;
         i=inputwin[MAIN_WIN].num_lists;
         inputwin[MAIN_WIN].lists[i].col=incol;
         inputwin[MAIN_WIN].lists[i].row=inrow;
         inputwin[MAIN_WIN].lists[i].maxlen=atoi(inputwin[win].inputs[1].text);
         inputwin[MAIN_WIN].lists[i].list_len=atoi(inputwin[win].inputs[2].text);
         inputwin[MAIN_WIN].lists[i].item=2;
         strcpy(inputwin[MAIN_WIN].lists[i].caption,inputwin[win].inputs[0].text);
         inputwin[MAIN_WIN].num_lists=i+1;

}

edit_form(int which,int type)
{
/*
                changes position or deletes an object

                opens a second window for input using inputwin structure

                draws buttons and text input/list boxes

                returns the button pressed and the updated text (in place)
                      and the updated list items

                includes event loop with redraw and resize

                needs the redraw for the main window added.

                could add keyboard short cuts

                uses redraw_inwin() for its window redraw

                type == 1 is input
                type == 2 is button
                type == 3 is list

*/

int done,i,in,q,win;
static int xpos=-1;
static int ypos= -1;
int col,row,inrow,incol;
char txt[20];

/* declarations for second window  */

Window my2window;
GC my2gc;
XEvent my2event;
XSizeHints my2hint;

        win = EDIT_WIN; /* window number in the structure */

        if(type == 1)
        {
             inrow=inputwin[MAIN_WIN].inputs[which].row;
             incol=inputwin[MAIN_WIN].inputs[which].col;
        }
        if(type == 2)
        {
             inrow=inputwin[MAIN_WIN].buttons[which].row;
             incol=inputwin[MAIN_WIN].buttons[which].col;
        }
        if(type == 3)
        {
             inrow=inputwin[MAIN_WIN].lists[which].row;
             incol=inputwin[MAIN_WIN].lists[which].col;
        }

        /* set the input boxes defaults */

        sprintf(inputwin[win].inputs[0].text,"%i",inrow);
        sprintf(inputwin[win].inputs[1].text,"%i",incol);
        
        if(debug)
        	printf("edit: inputs = %i\n",inputwin[win].num_inputs);
        
        if(xpos == -1 && ypos == -1)
        {
        	xpos = main_xcen - inputwin[win].xsize/2;
        	ypos = main_ycen - inputwin[win].ysize/2;
        	if(debug)
        		printf("button init pos: %i %i \n",xpos,ypos);
	}        	

         open_input_window(&my2window,&my2gc,xpos,ypos,inputwin[win].xsize,
                           inputwin[win].ysize,inputwin[win].title);


        /* event-reading loop to get cursor position */
        done = 0;
        while( done == 0) {
            /* read the next event */
            XNextEvent(mydisplay, &my2event);
            /* redraw main window */
            if(my2event.xany.window == mywindow)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                              {
                                        redraw(0);
                              }

                        break;

                        case ConfigureNotify:
                                set_box(my2event);
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                              XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* mouse-button presses in main beep */
                        case ButtonPress:
                                XBell(mydisplay,100);
                        break;

                        /* keyboard input in main is a beep */
                        case KeyPress:
                                XBell(mydisplay,100);
                        break;
                }
            }
            /* events for input window */
            if(my2event.xany.window == my2window)
            {
                switch(my2event.type){
                        /* repaint window on expose events */
                        case Expose:

                              if(my2event.xexpose.count == 0)
                                        redraw_inwin(my2window,my2gc,win);

                        break;

                        case LeaveNotify:
                               redraw_inwin(my2window,my2gc,win);
                               q = -1;
                        break;

                        case ConfigureNotify:
				save_position(my2event,&xpos,&ypos);
                        break;

                        case MotionNotify:
                                if(q != -1)
                                {
                                        if(q != which_button(my2event.xmotion.x,my2event.xmotion.y,win) )
                                        {
                                             redraw_inwin(my2window,my2gc,win);
                                             q = -1;
                                        }
                                  }
                        break;

                        /* process keyboard mapping changes */
                        case MappingNotify:
                        XRefreshKeyboardMapping(&my2event.xmapping);
                        break;

                        /* process mouse-button presses */
                        case ButtonPress:

                                    col=my2event.xmotion.x;
                                    row=my2event.xmotion.y;

                                    /* check input boxes */

                                    in = check_input(win,row,col);
                                    if( in > -1 )
                                    {
                                         if(get_input(my2window,my2gc,in,win) >= 0 )
                                         {
                                        /* code to do on input change goes next */


                                          }
                                          else
                                          {
                                               redraw_inwin(my2window,my2gc,win);
                                          }
                                      }

                                      q=check_button(my2window,my2gc,col,row,win);

                                      if(q == 0)
                                      {
                                       /* ok - reposition button  */

                                           edit_repos(win,which,type);

                                           done = 1;

                                      }
                                      if(q == 1)
                                      {
                                       /* delete button */

                                          delete(which,type);

                                          done = 1;
                                      }
                                      if( q == 2) /* apply button */
                                      {
                                      	   edit_repos(win,which,type);
                                      	   redraw(1);
                                      }

                        break;

                        /* process keyboard input */
                        case KeyPress:
                                i = XLookupString( &my2event.xkey, txt, 10, &mykey, 0);
                                /* if form key pressed code is needed it goes next */

                                if( mykey == XK_Escape ) done = 1;

                                if( (mykey == XK_Return) || (mykey == XK_KP_Enter) || (mykey == XK_Linefeed))
                                {
                                     edit_repos(win,which,type);
                                     done = 1;
                                }

                        break;
                } /* switch(my2event.type) */
            } /* if my window */
        } /* while(done == 0) */

        close_input_window(my2window,my2gc);

        redraw(1);

        return(done);
}

edit_repos(int win,int which,int type)
{
       if(type == 1)
       {
        inputwin[MAIN_WIN].inputs[which].row=atoi(inputwin[win].inputs[0].text);
        inputwin[MAIN_WIN].inputs[which].col=atoi(inputwin[win].inputs[1].text);
       }
       if(type == 2)
       {
        inputwin[MAIN_WIN].buttons[which].row=atoi(inputwin[win].inputs[0].text);
        inputwin[MAIN_WIN].buttons[which].col=atoi(inputwin[win].inputs[1].text);
       }
       if(type == 3)
       {
        inputwin[MAIN_WIN].lists[which].row=atoi(inputwin[win].inputs[0].text);
        inputwin[MAIN_WIN].lists[which].col=atoi(inputwin[win].inputs[1].text);
       }

}

delete(int which,int type)
{
int i,n;

         if(type == 1)
         {
            n=inputwin[MAIN_WIN].num_inputs-1;
            if(which == n)
            {
                inputwin[MAIN_WIN].num_inputs--;
                return;
            }
            for(i=which;i<n;i++)
            {
                inputwin[MAIN_WIN].inputs[i].row=inputwin[MAIN_WIN].inputs[i+1].row;
                inputwin[MAIN_WIN].inputs[i].col=inputwin[MAIN_WIN].inputs[i+1].col;
                inputwin[MAIN_WIN].inputs[i].maxlen=inputwin[MAIN_WIN].inputs[i+1].maxlen;
                strcpy(inputwin[MAIN_WIN].inputs[i].caption,inputwin[MAIN_WIN].inputs[i+1].caption);
	    }
            inputwin[MAIN_WIN].num_inputs--;
            return;
         }

         if(type == 2)
         {
            n=inputwin[MAIN_WIN].num_buttons-1;
            if(which == n)
            {
                inputwin[MAIN_WIN].num_buttons--;
                return;
            }
            for(i=which;i<n;i++)
            {
                inputwin[MAIN_WIN].buttons[i].row=inputwin[MAIN_WIN].buttons[i+1].row;
                inputwin[MAIN_WIN].buttons[i].col=inputwin[MAIN_WIN].buttons[i+1].col;
                strcpy(inputwin[MAIN_WIN].buttons[i].text,inputwin[MAIN_WIN].buttons[i+1].text);
	    }
            inputwin[MAIN_WIN].num_buttons--;
            return;
         }
         if(type == 3)
         {
            inputwin[MAIN_WIN].num_lists--;
            n=inputwin[MAIN_WIN].num_lists-1;
            if(which == n)
            {
                inputwin[MAIN_WIN].num_lists--;
                return;
            }
            for(i=which;i<n;i++)
            {
                inputwin[MAIN_WIN].lists[i].row=inputwin[MAIN_WIN].lists[i+1].row;
                inputwin[MAIN_WIN].lists[i].col=inputwin[MAIN_WIN].lists[i+1].col;
                inputwin[MAIN_WIN].lists[i].maxlen=inputwin[MAIN_WIN].lists[i+1].maxlen;
                inputwin[MAIN_WIN].lists[i].list_len=inputwin[MAIN_WIN].lists[i+1].list_len;
                strcpy(inputwin[MAIN_WIN].lists[i].caption,inputwin[MAIN_WIN].lists[i+1].caption);
	     }
             inputwin[MAIN_WIN].num_lists--;
             return;
         }

}

/*

        modified gfx routines follow (details)

*/

set_colors()
{
/*
     could be called from sinitg which is not currently used

*/

int status,i,ncmax;
char *colorname[]={ "blue","green","red","cyan","magenta","saddle brown"};

         ncmax=6;  /* set this to the number of colors listed above */

         num_mycolors = 1;
         mycolors[0]=myforeground;   /* always use black */

         if(depth == 1)
         {
                return(0);
         }

        mycolormap = DefaultColormap(mydisplay,myscreen);

        if(depth == 4)
        {
             for(i=0;i<4;i++)
             {

             status=XAllocNamedColor(mydisplay,mycolormap,colorname[i],&mycolor_def, &mycolor_rgb);

             if(status != 0 )
             {
                     mycolors[num_mycolors] = mycolor_def.pixel;
                     num_mycolors++;

              }
              }

         }


        if(depth >= 8)
        {
             for(i=0;i<ncmax;i++)
             {

                  status=XAllocNamedColor(mydisplay,mycolormap,colorname[i],&mycolor_def, &mycolor_rgb);

                  if(status != 0 )
                  {
                          mycolors[num_mycolors] = mycolor_def.pixel;
                          num_mycolors++;

                  }

             }

         }
}

set_box(XEvent event)
{
/*
        keeps track of the main window position and size
        
        pass it a ConfigureNotify event

*/
int xt,yt;
        xsize=event.xconfigure.width;
        ysize=event.xconfigure.height;
        xt=event.xconfigure.x;
        yt=event.xconfigure.y;
        if(xt != 0 && yt != 0)
        {
                main_xcen = xt + xsize/2;
                main_ycen = yt + ysize/2;
        }
        
        if(debug)
        {
        	printf("set_box: center = %i %i \n",main_xcen,main_ycen);
        }

        inputwin[MAIN_WIN].xsize=xsize;
        inputwin[MAIN_WIN].ysize=ysize;

}

save_position(XEvent event,int *xpos,int *ypos)
{
/*
	saves the current window position (corrected for WM border)
	if the ConfigureNotify event has position info
	
	set the defines to match your WM settings (4,21 work for me)
*/	
int x,y;

        x = event.xconfigure.x;
        y = event.xconfigure.y;
        if( x != 0 && y != 0)
        {
               *xpos = x - WM_BORDER_WIDTH;
               *ypos = y - WM_TITLEBAR_HEIGHT;
               if(debug)
                      printf("Button set pos: %i %i \n",*xpos,*ypos);
        }                        

}

get_size()
{
int x,y;

        XGetGeometry(mydisplay,mywindow,&root,
                     &x,&y,&xsize,&ysize,&border_width,&depth);

}

redraw(int flag)
{
int row,col,l;
char caption[30];

        if( flag == 1)
        {
                XClearWindow(mydisplay,mywindow);
        }

        redraw_inwin(mywindow,mygc,MAIN_WIN);

}


design_exit()
{

        /* termination */
        if(mydisplay != NULL)
        {
              XFreeGC(mydisplay, mygc);
              XDestroyWindow(mydisplay, mywindow);
              XCloseDisplay(mydisplay);
        }
        exit(0);

}


usage(cmd)
     char       *cmd;
{
  fprintf(stderr, "usage: %s [-g geometry] [-d display] [-v] [filename]\n", cmd);
}

